Package: huge
Type: Package
Title: High-Dimensional Undirected Graph Estimation
Version: 2.0.1
Authors@R: c(
    person("Haoming", "Jiang", role = "aut"),
    person("Xinyu", "Fei", role = "aut"),
    person("Han", "Liu", role = "aut"),
    person("Kathryn", "Roeder", role = "aut"),
    person("John", "Lafferty", role = "aut"),
    person("Larry", "Wasserman", role = "aut"),
    person("Xingguo", "Li", role = "aut"),
    person("Tuo", "Zhao", role = c("aut", "cre"), email = "tourzhao@gatech.edu")
  )
Maintainer: Tuo Zhao <tourzhao@gatech.edu>
Depends: R (>= 3.4.0)
Imports: Matrix, igraph, MASS, grDevices, graphics, methods, parallel,
        stats, utils, Rcpp
Suggests: testthat (>= 3.1.7)
LinkingTo: Rcpp
Description: Provides a general framework for
        high-dimensional undirected graph estimation. It integrates
        data preprocessing, neighborhood screening, graph estimation,
        and model selection techniques into a pipeline. In
        preprocessing stage, the nonparanormal(npn) transformation is
        applied to help relax the normality assumption. In the graph
        estimation stage, the graph structure is estimated by
        Meinshausen-Buhlmann graph estimation, the graphical lasso,
        or the TIGER (tuning-insensitive graph estimation and
        regression) method, and the first two can be further
        accelerated by the lossy screening rule preselecting the
        neighborhood of each variable by correlation thresholding. We
        target on high-dimensional data analysis usually d >> n, and
        the computation is memory-optimized using the sparse matrix
        output. We also provide a computationally efficient approach,
        correlation thresholding graph estimation. Three
        regularization/thresholding parameter selection methods are
        included in this package: (1)stability approach for
        regularization selection (2) rotation information criterion (3)
        extended Bayesian information criterion which is only available
        for the graphical lasso.
License: GPL-2
URL: https://github.com/Gatech-Flash/huge
BugReports: https://github.com/Gatech-Flash/huge/issues
Repository: CRAN
NeedsCompilation: yes
SystemRequirements: C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
Packaged: 2026-08-01 00:51:38 UTC; tourzhao
Author: Haoming Jiang [aut],
  Xinyu Fei [aut],
  Han Liu [aut],
  Kathryn Roeder [aut],
  John Lafferty [aut],
  Larry Wasserman [aut],
  Xingguo Li [aut],
  Tuo Zhao [aut, cre]
Date/Publication: 2026-08-04 10:30:34 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-08-04 13:04:28 UTC; unix
Archs: huge.so.dSYM
