# Generated by roxygen2: do not edit by hand

S3method(plot,huge)
S3method(plot,roc)
S3method(plot,select)
S3method(plot,sim)
S3method(print,huge)
S3method(print,roc)
S3method(print,select)
S3method(print,sim)
export(huge)
export(huge.ct)
export(huge.generator)
export(huge.glasso)
export(huge.inference)
export(huge.mb)
export(huge.npn)
export(huge.plot)
export(huge.roc)
export(huge.select)
export(huge.tiger)
importFrom(MASS,mvrnorm)
importFrom(Matrix,Matrix)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(grDevices,dev.off)
importFrom(grDevices,gray.colors)
importFrom(grDevices,postscript)
importFrom(graphics,image)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(methods,new)
importFrom(parallel,mclapply)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(utils,flush.console)
useDynLib(huge, .registration = TRUE)
