# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

RIC <- function(X, d, n, r, t) {
    .Call(`_huge_RIC`, X, d, n, r, t)
}

SFGen <- function(d0, d) {
    .Call(`_huge_SFGen`, d0, d)
}

SPMBscr <- function(S, lambda, nlambda, d, idx_scr, nscr) {
    .Call(`_huge_SPMBscr`, S, lambda, nlambda, d, idx_scr, nscr)
}

SPMBgraph <- function(S, lambda, nlambda, d) {
    .Call(`_huge_SPMBgraph`, S, lambda, nlambda, d)
}

SPMBgraphsqrt <- function(data, lambda, nlambda, d) {
    .Call(`_huge_SPMBgraphsqrt`, data, lambda, nlambda, d)
}

SPMBgraphsqrtFit <- function(input, lambda, nlambda, d, covariance_input, lambda_min_ratio) {
    .Call(`_huge_SPMBgraphsqrtFit`, input, lambda, nlambda, d, covariance_input, lambda_min_ratio)
}

hugeglasso <- function(S, lambda, scr, verbose, cov_output) {
    .Call(`_huge_hugeglasso`, S, lambda, scr, verbose, cov_output)
}

